import { encodeFunctionData, fromHex, isHex, keccak256, toBytes, } from 'viem';
import {} from 'viem/zksync';
import AccountFactoryAbi from './abis/AccountFactory.js';
import { SMART_ACCOUNT_FACTORY_ADDRESS } from './constants.js';
import {} from './types/call.js';
export function convertBigIntToString(value) {
    if (typeof value === 'bigint') {
        return value.toString();
    }
    else if (Array.isArray(value)) {
        return value.map(convertBigIntToString);
    }
    else if (typeof value === 'object' && value !== null) {
        return Object.fromEntries(Object.entries(value).map(([key, val]) => [
            key,
            convertBigIntToString(val),
        ]));
    }
    return value;
}
export async function getSmartAccountAddressFromInitialSigner(initialSigner, publicClient) {
    // Generate salt based off address
    const addressBytes = toBytes(initialSigner);
    const salt = keccak256(addressBytes);
    // Get the deployed account address
    const accountAddress = (await publicClient.readContract({
        address: SMART_ACCOUNT_FACTORY_ADDRESS,
        abi: AccountFactoryAbi,
        functionName: 'getAddressForSalt',
        args: [salt],
    }));
    return accountAddress;
}
export async function isSmartAccountDeployed(publicClient, address) {
    try {
        const bytecode = await publicClient.getCode({
            address: address,
        });
        return bytecode !== null && bytecode !== '0x' && bytecode !== undefined;
    }
    catch (error) {
        console.error('Error checking address:', error);
        return false;
    }
}
export function getInitializerCalldata(initialOwnerAddress, validatorAddress, initialCall) {
    return encodeFunctionData({
        abi: [
            {
                name: 'initialize',
                type: 'function',
                inputs: [
                    { name: 'initialK1Owner', type: 'address' },
                    { name: 'initialK1Validator', type: 'address' },
                    { name: 'modules', type: 'bytes[]' },
                    {
                        name: 'initCall',
                        type: 'tuple',
                        components: [
                            { name: 'target', type: 'address' },
                            { name: 'allowFailure', type: 'bool' },
                            { name: 'value', type: 'uint256' },
                            { name: 'callData', type: 'bytes' },
                        ],
                    },
                ],
                outputs: [],
                stateMutability: 'nonpayable',
            },
        ],
        functionName: 'initialize',
        args: [initialOwnerAddress, validatorAddress, [], initialCall],
    });
}
export function transformHexValues(transaction, keys) {
    if (!transaction)
        return;
    for (const key of keys) {
        if (isHex(transaction[key])) {
            transaction[key] = fromHex(transaction[key], 'bigint');
        }
    }
}
//# sourceMappingURL=utils.js.map